@@ -1,5 +1,5 @@
 setMethod("asJSON", "logical", function(x, collapse = TRUE, na = c("null", "string", "NA"),
-  auto_unbox = FALSE, keep_vec_names = FALSE, ...) {
+  auto_unbox = FALSE, keep_vec_names = FALSE, indent = NA_integer_, ...) {
 
   # shiny legacy exception
   if(isTRUE(keep_vec_names) && length(names(x))){
@@ -32,7 +32,7 @@ setMethod("asJSON", "logical", function(x, collapse = TRUE, na = c("null", "stri
 
   # collapse it
   if(collapse) {
-    collapse(tmp)
+    collapse(tmp, indent = indent)
   } else {
     tmp
   }
